<?php
/**
 * Plugin Name: Free Download for WooCommerce (EasyToolsWP)
 * Plugin URI: https://www.easytoolswp.com/plugins/woocommerce/free-download-button.zip
 * Description: Enable free downloads directly from product pages without going through the checkout process.
 * Version: 1.3
 * Author: EasyToolsWP
 */

if (!defined('ABSPATH')) {
    exit;
}

// Add custom meta box to product edit page
add_action('woocommerce_product_options_general_product_data', 'etwp_fdfw_add_free_download_checkbox');

function etwp_fdfw_add_free_download_checkbox() {
    woocommerce_wp_checkbox(array(
        'id' => 'etwp_fdfw_free_download',
        'label' => __('Free Download', 'easytoolswp'),
        'description' => __('Enable to allow free download of the product.', 'easytoolswp'),
    ));
}

// Save the checkbox value
add_action('woocommerce_process_product_meta', 'etwp_fdfw_save_free_download_checkbox');

function etwp_fdfw_save_free_download_checkbox($post_id) {
    $free_download = isset($_POST['etwp_fdfw_free_download']) ? 'yes' : 'no';
    update_post_meta($post_id, 'etwp_fdfw_free_download', $free_download);
}

// Customize product page to show download button instead of add to cart button and hide other elements
add_action('woocommerce_single_product_summary', 'etwp_fdfw_show_free_download_button', 30);

function etwp_fdfw_show_free_download_button() {
    global $product;

    if (get_post_meta($product->get_id(), 'etwp_fdfw_free_download', true) === 'yes') {
        remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30);
        remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_price', 10);

        // Hide other elements with CSS
        add_action('wp_footer', 'etwp_fdfw_hide_elements_with_css');
        
        // Get download files
        $downloads = $product->get_files();
        if ($downloads) {
            echo '<div class="woocommerce-free-downloads">';
            foreach ($downloads as $key => $file) {
                $count = get_post_meta($product->get_id(), 'etwp_fdfw_download_count_' . $key, true) ?: 0;
                echo '<a class="button product_type_simple add_to_cart_button etwp_fdfw_download_button" href="' . esc_url($file['file']) . '" data-product-id="' . $product->get_id() . '" data-download-key="' . $key . '">' . __('Download', 'easytoolswp') . '</a>';
                echo '<div class="etwp_fdfw_download_count">' . __('Downloads:', 'easytoolswp') . ' ' . $count . '</div>';
            }
            echo '</div>';
        }
    }
}

function etwp_fdfw_hide_elements_with_css() {
    if (is_product()) {
        echo '<style>
            .single_add_to_cart_button,
			.qty,
            .price
             {
                display: none !important;
            }
				#currency_switcher, .wapf, .etwp-product-sales-count{
                display: none !important;
            }

        </style>';
    }
}

// AJAX handler to count clicks on download button
add_action('wp_ajax_etwp_fdfw_count_download', 'etwp_fdfw_count_download');
add_action('wp_ajax_nopriv_etwp_fdfw_count_download', 'etwp_fdfw_count_download');

function etwp_fdfw_count_download() {
    $product_id = absint($_POST['product_id']);
    $download_key = sanitize_text_field($_POST['download_key']);
    $meta_key = 'etwp_fdfw_download_count_' . $download_key;

    $count = get_post_meta($product_id, $meta_key, true) ?: 0;
    $count++;
    update_post_meta($product_id, $meta_key, $count);

    wp_send_json_success(array('count' => $count));
}

// Enqueue scripts and localize for AJAX handling
add_action('wp_enqueue_scripts', 'etwp_fdfw_enqueue_scripts');

function etwp_fdfw_enqueue_scripts() {
    if (is_product()) {
        wp_enqueue_script('etwp_fdfw_script', plugin_dir_url(__FILE__) . 'etwp_fdfw_script.js', array('jquery'), null, true);
        wp_localize_script('etwp_fdfw_script', 'etwp_fdfw_ajax', array('ajax_url' => admin_url('admin-ajax.php')));
    }
}

// Add settings subpage for button text customization
add_action('admin_menu', 'etwp_fdfw_add_settings_subpage');

function etwp_fdfw_add_settings_subpage() {
    add_submenu_page(
        '',
        __('Free Download Settings', 'easytoolswp'),
        __('Free Download Settings', 'easytoolswp'),
        'manage_options',
        'easytoolswp_pro-settings',
        'etwp_fdfw_settings_page'
    );
}

function etwp_fdfw_settings_page() {
    ?>
    <div class="wrap">
        <h1><?php echo __('Free Download Settings', 'easytoolswp'); ?></h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('etwp_fdfw_settings_group');
            do_settings_sections('etwp_fdfw_settings_section');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Register settings and fields
add_action('admin_init', 'etwp_fdfw_register_settings');

function etwp_fdfw_register_settings() {
    register_setting('etwp_fdfw_settings_group', 'etwp_fdfw_button_text_option', array(
        'type' => 'string',
        'default' => 'button_text',
        'sanitize_callback' => 'sanitize_text_field',
    ));

    register_setting('etwp_fdfw_settings_group', 'etwp_fdfw_custom_button_text', array(
        'type' => 'string',
        'default' => '',
        'sanitize_callback' => 'sanitize_text_field',
    ));

    add_settings_section(
        'etwp_fdfw_settings_section',
        __('Button Text Options', 'easytoolswp'),
        'etwp_fdfw_settings_section_callback',
        'etwp_fdfw_settings_section'
    );

    add_settings_field(
        'etwp_fdfw_button_text_option',
        __('Button Text Option', 'easytoolswp'),
        'etwp_fdfw_button_text_option_callback',
        'etwp_fdfw_settings_section',
        'etwp_fdfw_settings_section'
    );

    add_settings_field(
        'etwp_fdfw_custom_button_text',
        __('Custom Button Text', 'easytoolswp'),
        'etwp_fdfw_custom_button_text_callback',
        'etwp_fdfw_settings_section',
        'etwp_fdfw_settings_section'
    );
}

function etwp_fdfw_settings_section_callback() {
    echo '<p>' . __('Choose the text for the download button.', 'easytoolswp') . '</p>';
}

function etwp_fdfw_button_text_option_callback() {
    $options = get_option('etwp_fdfw_button_text_option', 'button_text');
    ?>
    <select name="etwp_fdfw_button_text_option" id="etwp_fdfw_button_text_option">
        <option value="button_text" <?php selected($options, 'button_text'); ?>><?php _e('Button text', 'easytoolswp'); ?></option>
        <option value="filename" <?php selected($options, 'filename'); ?>><?php _e('Filename', 'easytoolswp'); ?></option>
        <option value="button_text_filename" <?php selected($options, 'button_text_filename'); ?>><?php _e('Button text + Filename', 'easytoolswp'); ?></option>
        <option value="filename_text" <?php selected($options, 'filename_text'); ?>><?php _e('Filename + Text', 'easytoolswp'); ?></option>
    </select>

    <br/>
    
    <label for="etwp_fdfw_custom_button_text"><?php _e('Custom Button Text', 'easytoolswp'); ?></label>
    <input type="text" id="etwp_fdfw_custom_button_text" name="etwp_fdfw_custom_button_text" value="<?php echo esc_attr(get_option('etwp_fdfw_custom_button_text', '')); ?>" />
    <?php
}

function etwp_fdfw_custom_button_text_callback() {
    $custom_button_text = get_option('etwp_fdfw_custom_button_text', '');
    ?>
    <input type="text" id="etwp_fdfw_custom_button_text" name="etwp_fdfw_custom_button_text" value="<?php echo esc_attr($custom_button_text); ?>" />
    <?php
}

// JavaScript for AJAX handling
add_action('wp_footer', 'etwp_fdfw_add_ajax_script');

function etwp_fdfw_add_ajax_script() {
    if (is_product()) {
        ?>
        <script type="text/javascript">
        jQuery(document).ready(function($) {
    var buttonOption = '<?php echo get_option('etwp_fdfw_button_text_option', 'button_text'); ?>';

    $('.etwp_fdfw_download_button').each(function() {
        var buttonText = $(this).text();
        $(this).on('click', function(e) {
            e.preventDefault();

            var button = $(this);
            var product_id = button.data('product-id');
            var download_key = button.data('download-key');

            $.post(etwp_fdfw_ajax.ajax_url, {
                action: 'etwp_fdfw_count_download',
                product_id: product_id,
                download_key: download_key
            }, function(response) {
                if (response.success) {
                    var countText = '<?php echo __('Downloads:', 'easytoolswp'); ?> ' + response.data.count;
                    if (buttonOption === 'filename') {
                        window.location.href = button.attr('href');
                    } else if (buttonOption === 'button_text_filename') {
                        button.text(buttonText + ' ' + button.attr('download'));
                        window.location.href = button.attr('href');
                    } else if (buttonOption === 'filename_text') {
                        button.text(buttonText + ' ' + button.attr('download'));
                        window.location.href = button.attr('href');
                    } else {
                        button.next('.etwp_fdfw_download_count').text(countText);
                        window.location.href = button.attr('href');
                    }
                }
            });
        });
    });
});

        </script>
        <?php
    }
}
?>